#include "graphhistory.h"

GraphHistory::GraphHistory()
{

}

QString GraphHistory::addRow(git_commit* commit)
{
    // graph
    QString ret = QString(" ").repeated(line.size());
    const git_oid* oid = git_commit_id(commit);

    // ## Lines for historic entries that bypass us:

    for (int i = 0; i < line.size(); ++i) {
        if (! git_oid_iszero(& line[i])) {
            ret[i] = '|';
        }
    }

    // ## The commit itself:

    // If this commit matches one of the wanted slots
    // render it in that position and clear the slot
    // Otherwise this is a new head - put it in a new slot

    // Does this commit rev match any of our wanted slots?
    int commitSlot = -1;
    for (int i = 0; i < line.size(); ++i) {
        if (git_oid_cmp(& line[i], oid) == 0) {
            // found at position i
            commitSlot = i;
            ret[i] = 'o';
            line[i] = git_oid{0}; // clear
        }
    }

    if (commitSlot == -1) {
        // This is a new head - put it in a new slot
        ret += "o";
        line.push_back(git_oid{0});
        commitSlot = line.size() - 1;
    }

    // ## Lines for parents of this commit:

    // Put parent revs in free slots
    int nParents = git_commit_parentcount(commit);
    for (int i = 0; i < nParents; ++i) {
        git_commit* parent = nullptr;
        int err = git_commit_parent(&parent, commit, i);
        if (err != 0) {
            // FIXME handle error!
            continue; // assume this parent didn't even exist
        }

        const git_oid* parentOid = git_commit_id(parent);

        // #### Find slot to render parent into
        // Possibilities:
        // Is this an existing parent or new parent?
        int parentSlot = -1;
        for (int j = 0; j < line.size(); ++j) {
            if (git_oid_cmp(&line[j], parentOid) == 0) {
                parentSlot = j;
                break;
            }
        }

        if (parentSlot == -1) {
            // Try to add it exactly below
            if (git_oid_iszero(& line[commitSlot])) {
                parentSlot = commitSlot;
            }
        }

        if (parentSlot == -1) {
            // Add it in the first free space
            for (int j = 0; j < line.size(); ++j) {
                if (git_oid_iszero(& line[j])) {
                    parentSlot = j;
                }
            }
        }

        if (parentSlot == -1) {
            // Add it to the end
            parentSlot = line.size();
            line.push_back(git_oid{0});
        }

        // Store parent OID for later matching
        line[parentSlot] = *parentOid;

        // #### If the line isn't direct, need to draw a new line
        if (parentSlot == commitSlot) {
            ret[commitSlot] = '0'; // o with a | through it...

        } else {
            // Draw line from commitSlot out to parentSlot, with a / or \ as appropriate
            ret[parentSlot] = '\\';

        }

    }

    // Flatten slots to display text
    return ret;
}
